"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("../application");
var layout_1 = require("../layout");
function makeWebsocketUrl(url) {
    if (url[0] === '/') {
        url = window.location.origin + url;
    }
    return url.replace("http", "ws");
}
var defaultEndPoint = {
    request: function (req, url, options) {
        return url;
    },
    websocket: function (url, options) {
        return url;
    },
    login: function (req, res, url, options) {
        return null;
    },
    feedback: function (req, res, url, options) {
    },
};
exports.default = {
    name: "fetch-addon",
    component: /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.router = null;
            _this.loginPromise = null;
            _this.enableLoginRecovery = true;
            _this.resolveUrl = null;
            return _this;
        }
        class_1.prototype.pluginWillMount = function (parameters) {
            application_1.Application.fetchAPI = this.fetchAPI.bind(this);
            application_1.Application.websocketAPI = this.websocketAPI.bind(this);
            this.resolveUrl = parameters.resolveUrl || this.resolveUrl;
        };
        class_1.prototype.resolveEndPoint = function (url, options) {
            return defaultEndPoint;
        };
        class_1.prototype.websocketAPI = function (url, options) {
            var endpoint = this.resolveUrl("ws", url) || defaultEndPoint;
            return new WebSocket(makeWebsocketUrl(endpoint.websocket(url, options)));
        };
        class_1.prototype.fetchAPI = function (url, options) {
            var _this = this;
            var hasLoginRecovery = this.enableLoginRecovery && (!options || !options.noCredentials);
            var endpoint = this.resolveUrl("http", url) || defaultEndPoint;
            var req = Object.assign({}, options);
            req.headers = __assign({ "Accept": "application/json" }, req.headers);
            if (req.body) {
                req.method = req.method || "POST";
                if (req.body instanceof File) {
                    req.headers["Content-Type"] = req.body.type;
                }
                else if (req.body instanceof Object) {
                    req.headers["Content-Type"] = "application/json";
                    req.body = JSON.stringify(req.body);
                }
                else if (!req.headers["Content-Type"]) {
                    req.headers["Content-Type"] = "text/plain";
                }
            }
            else {
                req.method = req.method || "GET";
            }
            var handleResponse = function (res) {
                return new Promise(function (resolve, reject) {
                    var respondError = function (error) {
                        reject({ status: res.status, headers: res.headers, error: error });
                    };
                    var contentType = res.headers && res.headers.get("content-type");
                    var isJson = contentType && contentType.toLowerCase().indexOf("application/json") !== -1;
                    if (res.status >= 200 && res.status < 300) {
                        if (endpoint.feedback) {
                            endpoint.feedback(req, res, url, options);
                        }
                        if (res.status === 204) {
                            resolve({ status: res.status, headers: res.headers, json: isJson ? {} : undefined });
                        }
                        else if (req.useBlob) {
                            return res.blob().then(function (blob) {
                                return resolve({ status: res.status, headers: res.headers, blob: blob });
                            }, respondError);
                        }
                        else {
                            return res.text().then(function (text) {
                                var json;
                                if (isJson) {
                                    try {
                                        json = isJson && JSON.parse(text);
                                    }
                                    catch (e) {
                                        return respondError(e);
                                    }
                                }
                                return resolve({ status: res.status, headers: res.headers, text: text, json: json });
                            }, respondError);
                        }
                    }
                    else if (res.status === 401 && hasLoginRecovery && endpoint.login) {
                        if (!_this.loginPromise) {
                            _this.loginPromise = endpoint.login(req, res, url, options);
                            if (!_this.loginPromise)
                                throw new Error();
                        }
                        _this.loginPromise.then(function () {
                            _this.loginPromise = null;
                            hasLoginRecovery = false;
                            resolve(fetch(endpoint.request(req, url, options), req).then(handleResponse, handleResponse));
                        }, function (error) {
                            _this.loginPromise = null;
                            _this.enableLoginRecovery = false;
                            respondError(error);
                        });
                    }
                    else if (isJson && res.json instanceof Function) {
                        return res.json().then(respondError, function () {
                            _this.enableLoginRecovery = false;
                            respondError(res.error);
                        });
                    }
                    else {
                        return respondError(res.error);
                    }
                });
            };
            return fetch(endpoint.request(req, url, options), req).then(handleResponse, handleResponse);
        };
        return class_1;
    }(layout_1.PluginInstance)),
};
