"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ApplicationInstance = /** @class */ (function () {
    function ApplicationInstance() {
    }
    ApplicationInstance.prototype.installFeatures = function (features, onlyFunction) {
        var _this = this;
        Object.keys(features).forEach(function (key) {
            if (!onlyFunction || (features[key] instanceof Function)) {
                _this[key] = features[key];
            }
        });
    };
    ApplicationInstance.prototype.injectAsProperty = function (clazz) {
        Object.defineProperty(clazz.prototype, "application", {
            value: exports.Application,
            writable: false,
            enumerable: false,
            configurable: false,
        });
    };
    ApplicationInstance.prototype.absoluteUrl = function (url) {
        if (url[0] === '/') {
            return window.location.origin + url;
        }
        return url;
    };
    ApplicationInstance.prototype.isHostedWindow = function () {
        return window.parent && window.parent !== window;
    };
    ApplicationInstance.prototype.redirectWindow = function (url) {
        url = this.absoluteUrl(url);
        if (this.isHostedWindow()) {
            window.parent.postMessage(JSON.stringify({ redirect: url, }), '*');
        }
        else {
            window.location = url;
        }
    };
    ApplicationInstance.prototype.openWindow = function (url) {
        url = this.absoluteUrl(url);
        if (this.isHostedWindow()) {
            window.parent.postMessage(JSON.stringify({ open: url, }), '*');
        }
        else {
            window.open(url);
        }
    };
    ApplicationInstance.prototype.reloadWindow = function () {
        if (this.isHostedWindow()) {
            window.location.reload();
        }
        else {
            window.reload();
        }
    };
    ApplicationInstance.prototype.setWindowTitle = function (title, icon) {
        if (this.isHostedWindow()) {
            window.parent.postMessage(JSON.stringify({ infos: { title: title, icon: icon } }), '*');
        }
        else {
            document.title = title;
        }
    };
    ApplicationInstance.prototype.setWindowMenu = function (menu, callbacks) {
        this.windowMenu = {
            menu: menu,
            callbacks: callbacks,
        };
        if (window.parent && window.parent !== window) {
            window.parent.postMessage(JSON.stringify({ menu: menu }), '*');
        }
    };
    ApplicationInstance.prototype.receiveHostMessage = function (msg) {
        if (msg.action && this.windowMenu && this.windowMenu.callbacks) {
            var callback = this.windowMenu.callbacks[msg.action];
            callback && callback(msg.action);
        }
    };
    return ApplicationInstance;
}());
exports.ApplicationInstance = ApplicationInstance;
window.addEventListener("keydown", function (e) {
    if (exports.Application.isHostedWindow()) {
        if (e.keyCode === 82 && e.ctrlKey) {
            exports.Application.reloadWindow();
            e.preventDefault();
            e.stopPropagation();
        }
    }
});
window.addEventListener("message", function (msg) {
    try {
        var data = JSON.parse(msg.data);
        if (msg.source === window.parent) {
            return exports.Application.receiveHostMessage(data);
        }
    }
    catch (e) { }
});
function extendApplication(features) {
    Object.keys(features).forEach(function (key) {
        Object.defineProperty(ApplicationInstance.prototype, key, {
            value: features[key],
            writable: false,
            enumerable: false,
            configurable: false,
        });
    });
}
exports.extendApplication = extendApplication;
exports.Application = new ApplicationInstance();
