"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Plugin_1 = require("./Plugin");
var Window_1 = require("./Window");
var windowsDocks_Key = "[react-application-frame]#windows-docks";
var PluginContext = /** @class */ (function () {
    function PluginContext() {
        var _this = this;
        this.pluginClasses = {};
        this.plugins = {};
        this.windowsDocks = {};
        this.windows = {};
        this.docks = {};
        this.focused = null;
        this.frame = null;
        this.uidGenerator = 0;
        this.mountPlugins = function () {
            var pluginNames = Object.keys(_this.pluginClasses);
            // Connect plugins
            pluginNames.forEach(function (name) {
                _this.pluginClasses[name].mount();
            });
            // Finalize plugins mount
            pluginNames.forEach(function (name) {
                _this.pluginClasses[name].didMount();
            });
        };
        this.unmountPlugins = function () {
            localStorage.setItem(windowsDocks_Key, JSON.stringify(_this.windowsDocks));
            Object.keys(_this.plugins).forEach(function (name) {
                var plugin = _this.plugins[name];
                plugin.pluginWillUnmount();
            });
        };
        try {
            var windowsDocks = JSON.parse(localStorage.getItem(windowsDocks_Key));
            if (windowsDocks && typeof windowsDocks === "object")
                this.windowsDocks = windowsDocks;
        }
        catch (e) { }
        window.addEventListener("beforeunload", this.unmountPlugins);
    }
    PluginContext.prototype.mapPlugin = function (name) {
        var pluginClass = this.pluginClasses[name];
        if (!pluginClass) {
            pluginClass = new Plugin_1.PluginClass(name, this);
            this.pluginClasses[name] = pluginClass;
        }
        return pluginClass;
    };
    PluginContext.prototype.installPlugin = function (description, parameters) {
        var pluginClass = this.mapPlugin(description.name);
        pluginClass.setup(description, parameters);
        pluginClass.mount();
        return pluginClass;
    };
    PluginContext.prototype.declarePlugin = function (description, parameters) {
        var pluginClass = this.mapPlugin(description.name);
        pluginClass.setup(description, parameters);
        return pluginClass;
    };
    PluginContext.prototype.mountPlugin = function (name) {
        var pluginClass = this.pluginClasses[name];
        pluginClass && pluginClass.mount();
        return pluginClass;
    };
    PluginContext.prototype.dockWindow = function (wnd, dockId, foreground) {
        this.frame && this.frame.attachWindow(wnd, dockId, foreground);
    };
    PluginContext.prototype.dettachWindow = function (wnd) {
        this.frame && this.frame.dettachWindow(wnd);
    };
    PluginContext.prototype.removeWindow = function (wnd) {
        this.dettachWindow(wnd);
        delete this.windows[wnd.id];
        wnd.close();
    };
    PluginContext.prototype.openSubWindow = function (windowClass, parent, options) {
        if (windowClass && parent) {
            var wnd = (options && options.openNew) ? null : this.findOneWindowByClass(windowClass);
            if (!wnd) {
                wnd = new Window_1.WindowInstance("#" + (this.uidGenerator++), windowClass, parent, parent.plugin, options);
            }
            else {
                options && wnd.updateOptions(options);
            }
            this.dockWindow(wnd, wnd.dockId, true);
        }
    };
    PluginContext.prototype.openPluginWindow = function (windowClass, plugin, options) {
        if (windowClass) {
            var wnd = (options && options.openNew) ? null : this.findOneWindowByClass(windowClass);
            if (!wnd) {
                wnd = new Window_1.WindowInstance("#" + (this.uidGenerator++), windowClass, null, plugin, options);
            }
            else {
                options && wnd.updateOptions(options);
            }
            this.dockWindow(wnd, wnd.dockId, true);
        }
    };
    PluginContext.prototype.closePluginWindows = function (windowClass, plugin) {
        var _this = this;
        var windows;
        if (windowClass)
            windows = this.findAllWindowsByClass(windowClass);
        else
            windows = this.findAllWindowsByPlugin(plugin);
        windows.forEach(function (wnd) { return _this.removeWindow(wnd); });
    };
    PluginContext.prototype.getWindowInstance = function (windowId) {
        return this.windows[windowId];
    };
    PluginContext.prototype.findOneWindowByClass = function (windowClass) {
        var windowId;
        for (windowId in this.windows) {
            if (this.windows[windowId].windowClass === windowClass) {
                return this.windows[windowId];
            }
        }
        return null;
    };
    PluginContext.prototype.findAllWindowsByPlugin = function (plugin) {
        var windows = [];
        if (plugin) {
            var windowId = void 0;
            for (windowId in this.windows) {
                if (this.windows[windowId].plugin === plugin) {
                    windows.push(this.windows[windowId]);
                }
            }
        }
        return windows;
    };
    PluginContext.prototype.findAllWindowsByClass = function (windowClass) {
        if (windowClass) {
            var windowId = void 0;
            var windows = [];
            for (windowId in this.windows) {
                if (this.windows[windowId].windowClass === windowClass) {
                    windows.push(this.windows[windowId]);
                }
            }
            return windows;
        }
        else {
            return this.windows.values();
        }
    };
    return PluginContext;
}());
exports.PluginContext = PluginContext;
