"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-use-before-define */
var listenable_1 = __importDefault(require("../modules/listenable"));
var Window_1 = require("./Window");
var application_1 = require("../application");
var PluginClass = /** @class */ (function () {
    function PluginClass(name, context) {
        this.name = name;
        this.context = context;
    }
    PluginClass.prototype.setup = function (desc, parameters) {
        this.parameters = parameters;
        this.component = desc.component || PluginInstance;
        this.export = desc.export;
        this.menu = desc.menu;
        // Setup dependencies
        if (desc.dependencies) {
            for (var _i = 0, _a = desc.dependencies; _i < _a.length; _i++) {
                var name_1 = _a[_i];
                this.addDependency(name_1);
            }
        }
        if (desc.import) {
            this.import = desc.import;
            for (var name_2 in desc.import) {
                this.addDependency(name_2);
            }
        }
        // Setup windows
        if (desc.windows) {
            for (var name_3 in desc.windows) {
                if (!this.windows)
                    this.windows = {};
                var window_1 = new Window_1.WindowClass(name_3, desc.windows[name_3], this);
                for (var key in window_1.parameters) {
                    var reference = window_1.parameters[key];
                    var _b = this.resolveValueReference(reference, key), pluginName = _b.pluginName, path = _b.path;
                    if (pluginName)
                        window_1.addLink(pluginName, key, path);
                    else
                        console.error("Parameter '" + key + "' of window '" + window_1.name + "' has invalid link:", path);
                }
                this.windows[name_3] = window_1;
            }
            this.addDependency("windows-frame");
        }
    };
    PluginClass.prototype.addUser = function (user) {
        if (!this.users)
            this.users = [];
        this.users.push(user);
        return this;
    };
    PluginClass.prototype.addDependency = function (name) {
        var pluginClass = this.context.mapPlugin(name);
        if (!this.dependencies)
            this.dependencies = [];
        var deps = this.dependencies;
        for (var i = 0; i < deps.length; i++) {
            if (deps[i] === pluginClass)
                return;
        }
        deps.push(pluginClass);
        return pluginClass.addUser(this);
    };
    PluginClass.prototype.raiseInvalid = function () {
        var msg = "The plugin '" + this.name + "' is invalid or missing";
        if (this.users) {
            msg += ", check dependencies at:";
            for (var _i = 0, _a = this.users; _i < _a.length; _i++) {
                var user = _a[_i];
                msg += "\n > plugin '" + user.name + "'";
            }
        }
        throw new Error(msg);
    };
    PluginClass.prototype.mount = function () {
        if (!this.instance) {
            if (!this.component)
                this.raiseInvalid();
            // Create instance
            this.instance = new (this.component)(this);
            this.context.plugins[this.name] = this.instance;
            // Call will mount
            this.instance.pluginWillMount(this.parameters);
            // Mount all dependencies
            if (this.dependencies) {
                for (var _i = 0, _a = this.dependencies; _i < _a.length; _i++) {
                    var dep = _a[_i];
                    dep.mount();
                }
            }
            // Notify all user plugins
            if (this.users) {
                for (var _b = 0, _c = this.users; _b < _c.length; _b++) {
                    var user = _c[_b];
                    user.promptMount();
                }
            }
            // Try to finalize mounting
            this.promptMount();
        }
    };
    PluginClass.prototype.promptMount = function () {
        if (this.instance && !this.isMounted) {
            var deps = this.dependencies;
            if (deps) {
                for (var i = 0; i < deps.length; i++) {
                    if (!deps[i].instance)
                        return;
                }
            }
            this.didMount();
        }
    };
    PluginClass.prototype.didMount = function () {
        if (this.instance && !this.isMounted) {
            // Process import
            for (var name_4 in this.import) {
                var ref = this.import[name_4];
                var plugin = this.context.plugins[name_4];
                var pluginExport = plugin[".class"].export;
                if (pluginExport) {
                    if (ref === true) {
                        for (var _i = 0, pluginExport_1 = pluginExport; _i < pluginExport_1.length; _i++) {
                            var key = pluginExport_1[_i];
                            this.instance[key] = plugin[key];
                        }
                    }
                    else if (typeof ref === "string") {
                        this.instance[ref] = plugin;
                    }
                }
                else if (typeof ref === "string") {
                    this.instance[ref] = plugin;
                }
            }
            this.instance.pluginDidMount({});
            this.isMounted = true;
        }
    };
    PluginClass.prototype.resolveValueReference = function (reference, key) {
        var pluginName, path;
        if (reference === true) {
            pluginName = this.name;
            path = key;
        }
        else if (typeof reference === "string") {
            var parts = reference.split("/");
            if (parts.length > 1) {
                pluginName = parts[0] ? parts[0] : this.name;
                path = parts[1];
            }
            else {
                pluginName = this.name;
                path = reference;
            }
        }
        return { pluginName: pluginName, path: path };
    };
    return PluginClass;
}());
exports.PluginClass = PluginClass;
var PluginInstance = /** @class */ (function (_super) {
    __extends(PluginInstance, _super);
    function PluginInstance(pluginClass) {
        var _this = _super.call(this) || this;
        _this[".class"] = pluginClass;
        if (pluginClass.windows) {
            _this.openWindow = function (windowName, options) {
                application_1.Application.layout.openPluginWindow(pluginClass.windows[windowName], this, options);
            };
            _this.closeWindow = function (windowName) {
                application_1.Application.layout.closePluginWindows(pluginClass.windows[windowName], this);
            };
            _this.closeAllWindow = function () {
                application_1.Application.layout.closePluginWindows(null, this);
            };
        }
        return _this;
    }
    // Life Cycle management functions
    PluginInstance.prototype.pluginWillMount = function (parameters) { };
    PluginInstance.prototype.pluginDidMount = function (parameters) { };
    PluginInstance.prototype.pluginWillUnmount = function () { };
    return PluginInstance;
}(listenable_1.default));
exports.PluginInstance = PluginInstance;
