import React, { Component } from "react";
import Listenable from "../modules/listenable";
import { PluginClass, PluginInstance } from "./Plugin";
export declare type WindowOptions = {
    title?: string;
    icon?: Function;
    dockId?: string;
    parameters?: {
        [key: string]: any;
    };
    openNew?: boolean;
};
export declare type ParameterLink = {
    [key: string]: string;
};
export declare class WindowClass {
    classId: string;
    name: string;
    overflow: string;
    keepAlive: boolean;
    pluginClass: PluginClass;
    component: typeof WindowComponent;
    parameters: {
        [key: string]: any;
    };
    defaultTitle: string;
    defaultDockId: string;
    defaultParameters: Object;
    defaultIcon: Function;
    windows: {
        [key: string]: WindowClass;
    };
    links: {
        [key: string]: ParameterLink;
    };
    constructor(name: string, desc: any, pluginClass: PluginClass);
    getDefaultDockId(): string;
    setDefaultDockId(defaultDockId: any): void;
    addLink(pluginName: string, key: string, path: string): void;
    updateParametersFor(instance: any, pluginName: any, bind?: boolean): void;
    connectParameters(instance: WindowInstance): void;
    disconnectParameters(instance: WindowInstance): void;
}
export declare class WindowInstance extends Listenable {
    id: string;
    windowClass: WindowClass;
    window: WindowComponent;
    parent: WindowInstance;
    plugin: PluginInstance;
    container: WindowContainer;
    node: HTMLElement;
    dockId: string;
    title: string;
    icon: Function;
    parameters: {
        [key: string]: any;
    };
    animation: any;
    constructor(windowId: string, windowClass: WindowClass, parent: WindowInstance, plugin: PluginInstance, options: WindowOptions);
    readonly hasFocus: boolean;
    setWindow(window: WindowComponent): void;
    updateParams: (plugin: any, prevState: any) => void;
    updateOptions(options: WindowOptions): void;
    updateTitle(animation?: any): void;
    close(): void;
    render(): void;
}
export declare type PropsType = {
    current: WindowInstance;
    className: string;
    style: any;
};
export declare class WindowContainer extends React.Component {
    props: PropsType;
    instance: WindowInstance;
    componentDidMount(): void;
    componentWillReceiveProps(nextProps: any): void;
    componentWillUnmount(): void;
    mountWindow(instance: WindowInstance): void;
    unmountWindow(): void;
    width(): number;
    height(): number;
    focus: (e: any) => void;
    handle: (type: string) => (e: any) => void;
    render(): JSX.Element;
}
export declare class WindowComponent extends Component {
    instance: WindowInstance;
    plugin: PluginInstance;
    constructor(props: any);
    isWindow(): boolean;
    addEventListener(): void;
    removeEventListener(): void;
    openWindow(windowClassID: string, options: WindowOptions): void;
    closeWindow(): void;
    updateWindowTitle(animation: Object): void;
    renderWindowIcon(): any;
    renderWindowTitle(): JSX.Element;
}
