"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-use-before-define */
var react_1 = __importStar(require("react"));
var react_dom_1 = __importDefault(require("react-dom"));
var listenable_1 = __importDefault(require("../modules/listenable"));
var application_1 = require("../application");
var utils_1 = require("../utils");
var WindowClass = /** @class */ (function () {
    function WindowClass(name, desc, pluginClass) {
        for (var key in desc)
            this[key] = desc[key];
        this.classId = pluginClass.name + ":" + name;
        this.name = name;
        this.overflow = desc.overflow || "auto";
        this.pluginClass = pluginClass;
        console.assert(utils_1.isInheritedOf(this.component, WindowComponent), "Window '", this.name, "' shall be based on WindowComponent");
    }
    WindowClass.prototype.getDefaultDockId = function () {
        var windowsDocks = this.pluginClass.context.windowsDocks;
        return windowsDocks[this.classId] || this.defaultDockId;
    };
    WindowClass.prototype.setDefaultDockId = function (defaultDockId) {
        var windowsDocks = this.pluginClass.context.windowsDocks;
        if (this.defaultDockId !== defaultDockId) {
            windowsDocks[this.classId] = defaultDockId;
        }
        else {
            delete windowsDocks[this.classId];
        }
    };
    WindowClass.prototype.addLink = function (pluginName, key, path) {
        if (!this.links)
            this.links = {};
        if (!this.links[pluginName])
            this.links[pluginName] = {};
        this.links[pluginName][key] = path;
    };
    WindowClass.prototype.updateParametersFor = function (instance, pluginName, bind) {
        if (bind === void 0) { bind = false; }
        var plugins = application_1.Application.layout.plugins;
        var plugInstance = plugins[pluginName];
        if (plugInstance) {
            var plugLinks = this.links[pluginName];
            bind && plugInstance.addEventListener(Object.keys(plugLinks), instance.updateParams);
            for (var key in plugLinks) {
                var path = plugLinks[key];
                instance.parameters[key] = plugInstance[path];
            }
        }
    };
    WindowClass.prototype.connectParameters = function (instance) {
        if (this.links) {
            for (var pluginName in this.links) {
                this.updateParametersFor(instance, pluginName, true);
            }
        }
    };
    WindowClass.prototype.disconnectParameters = function (instance) {
        if (this.links) {
            var plugins = application_1.Application.layout.plugins;
            for (var pluginName in this.links) {
                var plugInstance = plugins[pluginName];
                plugInstance && plugInstance.removeEventListener(instance.updateParams);
            }
        }
    };
    return WindowClass;
}());
exports.WindowClass = WindowClass;
var WindowInstance = /** @class */ (function (_super) {
    __extends(WindowInstance, _super);
    function WindowInstance(windowId, windowClass, parent, plugin, options) {
        var _this = _super.call(this) || this;
        _this.updateParams = function (plugin, prevState) {
            _this.windowClass.updateParametersFor(_this, plugin[".class"].name);
            _this.render();
        };
        _this.id = windowId;
        _this.windowClass = windowClass;
        _this.parent = parent;
        _this.plugin = plugin;
        _this.node = document.createElement("div");
        _this.node.className = "position-relative width-100 height-100 overflow-" + windowClass.overflow;
        application_1.Application.layout.windows[windowId] = _this;
        _this.title = windowClass.defaultTitle;
        _this.icon = windowClass.defaultIcon;
        _this.dockId = windowClass.getDefaultDockId();
        _this.parameters = {
            instance: _this,
            onChange: function (parameters) { return _this.updateOptions({ parameters: parameters }); },
        };
        _this.windowClass.connectParameters(_this);
        _this.updateOptions(options);
        return _this;
    }
    Object.defineProperty(WindowInstance.prototype, "hasFocus", {
        get: function () {
            return application_1.Application.layout.focused === this;
        },
        enumerable: true,
        configurable: true
    });
    WindowInstance.prototype.setWindow = function (window) {
        this.window = window;
        this.executeEvent("update");
    };
    WindowInstance.prototype.updateOptions = function (options) {
        if (options) {
            if (options.title)
                this.title = options.title;
            if (options.icon)
                this.icon = options.icon;
            if (options.dockId)
                this.dockId = options.dockId;
            if (options.parameters) {
                this.parameters = __assign({}, this.parameters, options.parameters);
            }
        }
        this.render();
    };
    WindowInstance.prototype.updateTitle = function (animation) {
        // Clear previous animation
        if (this.animation) {
            var timer = this.animation.timer;
            if (timer)
                clearTimeout(timer);
            this.animation = undefined;
        }
        // Prepare new animation
        if (animation) {
            var duration = animation.duration;
            if (duration) {
                animation.timer = setTimeout(this.updateTitle.bind(this), duration);
            }
            this.animation = animation;
        }
        this.dispatchEvent("update");
    };
    WindowInstance.prototype.close = function () {
        this.windowClass.disconnectParameters(this);
        react_dom_1.default.unmountComponentAtNode(this.node);
    };
    WindowInstance.prototype.render = function () {
        var frame = application_1.Application.layout.frame;
        var parentComponent = frame && frame.getFrameComponent();
        if (parentComponent) {
            react_dom_1.default.unstable_renderSubtreeIntoContainer(parentComponent, react_1.default.createElement(this.windowClass.component, this.parameters), this.node);
        }
    };
    return WindowInstance;
}(listenable_1.default));
exports.WindowInstance = WindowInstance;
var WindowContainer = /** @class */ (function (_super) {
    __extends(WindowContainer, _super);
    function WindowContainer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.focus = function (e) {
            var prev_focused = application_1.Application.layout.focused;
            if (prev_focused !== _this.instance) {
                application_1.Application.layout.focused = _this.instance;
                if (prev_focused) {
                    prev_focused.executeEvent("blur", e);
                    prev_focused.dispatchEvent("update");
                }
                if (_this.instance) {
                    _this.instance.executeEvent("focus", e);
                    _this.instance.updateTitle();
                }
            }
        };
        _this.handle = function (type) { return function (e) {
            _this.instance && _this.instance.executeEvent(type, e);
        }; };
        return _this;
    }
    WindowContainer.prototype.componentDidMount = function () {
        this.mountWindow(this.props.current);
    };
    WindowContainer.prototype.componentWillReceiveProps = function (nextProps) {
        if (this.props.current !== nextProps.current) {
            this.unmountWindow();
            this.mountWindow(nextProps.current);
        }
    };
    WindowContainer.prototype.componentWillUnmount = function () {
        this.unmountWindow();
    };
    WindowContainer.prototype.mountWindow = function (instance) {
        this.instance = instance;
        if (instance) {
            this.refs.root.appendChild(instance.node);
            instance.container = this;
        }
    };
    WindowContainer.prototype.unmountWindow = function () {
        if (this.instance && this.instance.container === this) {
            this.refs.root.removeChild(this.instance.node);
            this.instance.container = null;
        }
    };
    WindowContainer.prototype.width = function () {
        return this.refs.root && this.refs.root.clientWidth;
    };
    WindowContainer.prototype.height = function () {
        return this.refs.root && this.refs.root.clientHeight;
    };
    WindowContainer.prototype.render = function () {
        var _a = this.props, className = _a.className, style = _a.style;
        return (react_1.default.createElement("div", { ref: "root", tabIndex: 1, className: className, style: style, onKeyDown: this.handle("keydown"), onKeyPress: this.handle("keypress"), onKeyUp: this.handle("keyup"), onFocus: this.focus }));
    };
    return WindowContainer;
}(react_1.default.Component));
exports.WindowContainer = WindowContainer;
var WindowComponent = /** @class */ (function (_super) {
    __extends(WindowComponent, _super);
    function WindowComponent(props) {
        var _this = _super.call(this, props) || this;
        _this.instance = props.instance;
        _this.plugin = props.instance.plugin;
        _this.instance.setWindow(_this);
        return _this;
    }
    WindowComponent.prototype.isWindow = function () {
        return true;
    };
    WindowComponent.prototype.addEventListener = function () {
        this.instance.addEventListener.apply(this.instance, arguments);
    };
    WindowComponent.prototype.removeEventListener = function () {
        this.instance.removeEventListener.apply(this.instance, arguments);
    };
    WindowComponent.prototype.openWindow = function (windowClassID, options) {
        var windowClass = this.instance.windowClass;
        application_1.Application.layout.openSubWindow(windowClass.windows[windowClassID], this.instance, options);
    };
    WindowComponent.prototype.closeWindow = function () {
        application_1.Application.layout.removeWindow(this.instance);
    };
    WindowComponent.prototype.updateWindowTitle = function (animation) {
        this.instance.updateTitle(animation);
    };
    WindowComponent.prototype.renderWindowIcon = function () {
        var icon = this.instance.icon;
        if (icon instanceof Function)
            return icon();
        else if (react_1.default.isValidElement(icon))
            return icon;
        else
            return null;
    };
    WindowComponent.prototype.renderWindowTitle = function () {
        return react_1.default.createElement(react_1.default.Fragment, null,
            this.renderWindowIcon(),
            this.instance.title);
    };
    return WindowComponent;
}(react_1.Component));
exports.WindowComponent = WindowComponent;
