"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CallSite = /** @class */ (function () {
    function CallSite(self, proc, args) {
        this.status = -1;
        this.self = self;
        this.proc = proc;
        this.args = args;
    }
    CallSite.prototype.containOf = function (proc) {
        for (var c = this; c; c = c.back) {
            if (c.proc === proc && c.status < 2) {
                return c;
            }
        }
    };
    CallSite.prototype.abortAllOf = function (proc) {
        for (var c = this; c; c = c.back) {
            if (c.proc === proc && c.status < 2) {
                this.abort("aborted");
            }
        }
    };
    CallSite.prototype.abort = function (data) {
        if (this.status < 2) {
            this.status = 3;
            this._data = data;
            this._reject && this._reject(data);
            var nextCall = this.next;
            if (nextCall) {
                this.next.back = null;
                this.next = null;
                nextCall._do();
            }
        }
        else if (this.status < 3) {
            throw new Error("'abort/complete' cannot applied twice on CallSite");
        }
    };
    CallSite.prototype.complete = function (data) {
        if (this.status < 2) {
            this.status = 2;
            this._data = data;
            this._resolve && this._resolve(data);
            var nextCall = this.next;
            if (nextCall) {
                this.next.back = null;
                this.next = null;
                nextCall._do();
            }
        }
        else if (this.status < 3) {
            throw new Error("'abort/complete' cannot applied twice on CallSite");
        }
    };
    CallSite.prototype.when = function () {
        var _this = this;
        if (!this._when) {
            switch (this.status) {
                case 2:
                    this._when = Promise.resolve(this._data);
                    break;
                case 3:
                    this._when = Promise.reject(this._data);
                    break;
                default:
                    this._when = new Promise(function (_resolve, _reject) {
                        _this._resolve = _resolve;
                        _this._reject = _reject;
                    });
            }
        }
        return this._when;
    };
    CallSite.prototype._do = function () {
        var _this = this;
        if (this.status < 1) {
            this.status = 1;
            try {
                var result = this.proc.apply(this.self, this.args);
                if (result instanceof Promise) {
                    result.then(function (data) {
                        _this.complete(data);
                    }, function (e) {
                        _this.abort(e);
                    });
                }
                else {
                    this.complete(result);
                }
            }
            catch (e) {
                this.abort(e);
            }
        }
        else {
            throw new Error("'_do' cannot applied twice on CallSite");
        }
    };
    CallSite.prototype.execute = function (after) {
        if (this.status < 0) {
            this.status = 0;
            if (after) {
                if (after.next) {
                    this.next = after.next;
                    this.next.back = this;
                }
                if (after.status < 2 || after.back) {
                    this.back = after;
                    after.next = this;
                }
            }
            if (!this.back) {
                this._do();
            }
        }
        else {
            throw new Error("'execute' cannot applied twice on CallSite");
        }
    };
    return CallSite;
}());
exports.CallSite = CallSite;
var CallChain = /** @class */ (function () {
    function CallChain() {
    }
    CallChain.prototype.call = function (self, proc, args) {
        var call = new CallSite(self, proc, args);
        call.execute(this.tos);
        this.tos = call;
        return call;
    };
    CallChain.prototype.abortAllOf = function (proc) {
        return this.tos && this.tos.abortAllOf(proc);
    };
    CallChain.prototype.containOf = function (proc) {
        return this.tos && this.tos.containOf(proc);
    };
    return CallChain;
}());
exports.default = CallChain;
