"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dispatcheds = null;
var Listenable = /** @class */ (function () {
    function Listenable() {
    }
    Listenable.prototype.addEventListener = function () {
        // Prepare listeners array
        var listeners = this[".listeners"];
        if (!listeners)
            this[".listeners"] = listeners = [];
        else if (dispatcheds === listeners)
            this[".listeners"] = listeners = listeners.slice();
        // function addEventListener(callback: Function)
        //  - object.addEventListener(eventName, (data: any) => {})
        //  - object.addEventListener("change", (data: Listenable, prevState: Object) => {})
        //  - object.addEventListener(["field0","field1"], (data: Listenable, prevState: Object) => {})
        if (arguments.length === 1) {
            if (!arguments[0])
                return false;
            listeners.push(arguments[0]);
            listeners.push(null);
            return true;
        }
        // function addEventListener(type: string|Array|null, callback: Function)
        //  - object.addEventListener((type: string, data: any) => {})
        else if (arguments.length === 2) {
            if (!arguments[1])
                return false;
            listeners.push(arguments[1]);
            listeners.push(arguments[0]);
            return true;
        }
    };
    Listenable.prototype.removeEventListener = function () {
        // Prepare listeners array
        var listeners = this[".listeners"];
        if (!listeners)
            return false;
        else if (dispatcheds === listeners)
            this[".listeners"] = listeners = listeners.slice();
        // a. Remove all listeners of the given callback
        if (arguments.length === 1) {
            var callback = arguments[0];
            var index = 0;
            while (true) {
                index = listeners.indexOf(callback, index);
                if (index >= 0) {
                    listeners.splice(index, 2);
                    index++;
                }
                else
                    return true;
            }
        }
        // b. Remove one listener of the given eventType+callback
        else if (arguments.length === 2) {
            var eventType = arguments[0];
            var callback = arguments[1];
            var index = 0;
            while (true) {
                index = listeners.indexOf(callback, index);
                if (index >= 0) {
                    if (listeners[index + 1] === eventType) {
                        listeners.splice(index, 2);
                        return true;
                    }
                    index++;
                }
                else {
                    console.warn("removeEventListener(" + eventType + ") failed on:", this);
                    return false;
                }
            }
        }
        return false;
    };
    // Add object state listener, which will be called when a 'setState' is applied
    // > When return Promise: 'addStateListener' shall be retry on the future provided listenable
    // > When return Listenable: the truly listened object is returned, and 'removeFieldsListener' shall be applied on this returned object. 
    Listenable.prototype.synchronizeStateInContext = function (contexts, // Collection of data, the listenable can access to it environment variable by key
    writable, // true when a writable listenable is required for the resulting listenable
    fields, // fields required in the resulting listenable
    previousResult // give the previous listenable returned by this method (can help to optimize)
    ) {
        return this;
    };
    // Add object state listener, which will be called when a 'setState' is applied
    // > When return Promise: 'addStateListener' shall be retry on the future provided listenable
    // > When return Listenable: the truly listened object is returned, and 'removeFieldsListener' shall be applied on this returned object. 
    Listenable.prototype.addStateListener = function (callback, fields) {
        return this.addEventListener("change", callback);
    };
    // Remove object state listener
    Listenable.prototype.removeStateListener = function (callback) {
        return this.removeEventListener("change", callback);
    };
    Listenable.prototype.setState = function () {
        if (arguments.length === 0) {
            if (!this[".events"]) {
                this[".events"] = [{}];
                setTimeout(event_dispatcher.bind(this));
            }
            else if (!this[".events"][0])
                this[".events"][0] = {};
        }
        else if (arguments.length === 1) {
            for (var key in arguments[0]) {
                dispatchStateEvent.call(this, key, arguments[0][key]);
            }
        }
        else if (arguments.length === 2) {
            dispatchStateEvent.call(this, arguments[0], arguments[1]);
        }
    };
    Listenable.prototype.dispatchEvent = function (type, data) {
        // Prepare events slot
        if (!this[".events"]) {
            this[".events"] = [null];
            setTimeout(event_dispatcher.bind(this));
        }
        // Push event
        if (type === "change") {
            this[".events"][0] = Object.assign(this[".events"][0] || {}, data);
        }
        else {
            this[".events"].push(type);
            this[".events"].push(data);
        }
    };
    Listenable.prototype.executeEvent = function (type, data) {
        var listeners = this[".listeners"];
        var count = listeners ? listeners.length : 0;
        if (!count)
            return;
        // Retain listeners array
        if (dispatcheds === null)
            dispatcheds = listeners;
        else
            listeners = listeners.slice();
        // Execute listeners
        var promises = [];
        for (var k = 1; k < count; k += 2) {
            try {
                var result = void 0;
                var ltype = listeners[k];
                if (!ltype)
                    result = listeners[k - 1](type, data, this);
                else if (ltype === type)
                    result = listeners[k - 1](data, this);
                if (result instanceof Promise)
                    promises.push(result);
            }
            catch (e) {
                console.error(e);
            }
        }
        // Release listeners array
        if (dispatcheds === listeners)
            dispatcheds = null;
        return Promise.all(promises);
    };
    Listenable.setState = function () { };
    return Listenable;
}());
exports.default = Listenable;
Listenable.setState = Listenable.prototype.setState;
function dispatchStateEvent(key, value) {
    var _a;
    if (this[key] !== value) {
        // Prepare events slot
        if (!this[".events"]) {
            this[".events"] = [null];
            setTimeout(event_dispatcher.bind(this));
        }
        // Update change event
        var prevState = this[".events"][0];
        if (!prevState)
            this[".events"][0] = (_a = {}, _a[key] = value, _a);
        else if (!prevState.hasOwnProperty(key))
            prevState[key] = this[key];
        // Update object
        this[key] = value;
    }
}
function event_dispatcher() {
    var listeners = this[".listeners"];
    var events = this[".events"];
    var count = listeners ? listeners.length : 0;
    this[".events"] = null;
    if (!count)
        return;
    // Retain listeners array
    if (dispatcheds === null)
        dispatcheds = listeners;
    else
        listeners = listeners.slice();
    // Dispatch change event
    var prevState = events[0];
    if (prevState) {
        for (var k = 1; k < count; k += 2) {
            try {
                var ltype = listeners[k];
                if (!ltype) {
                    listeners[k - 1]("change", this, prevState);
                }
                else if (ltype === "change") {
                    listeners[k - 1](this, prevState);
                }
                else if (Array.isArray(ltype)) {
                    for (var _i = 0, ltype_1 = ltype; _i < ltype_1.length; _i++) {
                        var key = ltype_1[_i];
                        if (prevState.hasOwnProperty(key)) {
                            listeners[k - 1](this, prevState);
                            break;
                        }
                    }
                }
            }
            catch (e) {
                console.error(e);
            }
        }
    }
    // Dispatch other events
    for (var i = 2; i < events.length; i += 2) {
        var type = events[i - 1];
        var data = events[i];
        for (var k = 1; k < count; k += 2) {
            try {
                var ltype = listeners[k];
                if (!ltype)
                    listeners[k - 1](type, data, this);
                else if (ltype === type)
                    listeners[k - 1](data, this);
            }
            catch (e) {
                console.error(e);
            }
        }
    }
    // Release listeners array
    if (dispatcheds === listeners)
        dispatcheds = null;
}
