"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var listenable_1 = __importDefault(require("./listenable"));
var uidGenerator = window.localStorage.getItem("#uidGenerator") | 0;
function generateUid() {
    var uid = "obj#" + (uidGenerator++);
    window.localStorage.setItem("#uidGenerator", uidGenerator.toString());
    return uid;
}
exports.globalStorage = {};
exports.storableClasses = {};
var Storable = /** @class */ (function (_super) {
    __extends(Storable, _super);
    function Storable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Storable.prototype.storableShouldInitiate = function (data) {
    };
    Storable.prototype.storableShouldSerialize = function () {
        return null;
    };
    Storable.prototype.storableWillUnmount = function () {
    };
    Storable.prototype.readStorageData = function () {
        try {
            var bytes = window.localStorage.getItem(this.$$storeUid);
            if (bytes)
                return JSON.parse(bytes);
        }
        catch (e) {
            window.localStorage.removeItem(this.$$storeUid);
        }
        return null;
    };
    Storable.prototype.writeStorageData = function (data) {
        if (!data[".className"])
            throw new Error("Invalid storable record");
        try {
            var bytes = JSON.stringify(data);
            window.localStorage.setItem(this.$$storeUid, bytes);
            return true;
        }
        catch (e) { }
        return false;
    };
    Storable.prototype.patchStorageData = function (data) {
        try {
            var bytes = JSON.stringify(Object.assign(this.readStorageData(), data));
            window.localStorage.setItem(this.$$storeUid, bytes);
            return true;
        }
        catch (e) { }
        return false;
    };
    Storable.prototype.setStorageData = function (value) {
        var data = this.readStorageData();
        var hasChange = false;
        if (arguments.length === 1) {
            for (var key in value) {
                if (data[key] !== value[key]) {
                    data[key] = value[key];
                    hasChange = true;
                }
            }
        }
        else if (arguments.length === 2) {
            if (data[value] !== arguments[1]) {
                data[value] = arguments[1];
                hasChange = true;
            }
        }
        hasChange && this.writeStorageData(data);
    };
    Storable.prototype.forceSave = function () {
        try {
            var data = this.storableShouldSerialize();
            if (data) {
                data[".className"] = this.constructor[".className"];
                window.localStorage.setItem(this.$$storeUid, JSON.stringify(data));
            }
            return true;
        }
        catch (e) {
            window.localStorage.setItem(this.$$storeUid, e.toString());
            console.error(e);
        }
        return false;
    };
    Storable.prototype.terminateState = function () {
        this[".listeners"] = null;
        this.dispatchEvent("terminate");
        this.forceSave();
        this.storableWillUnmount();
        delete exports.globalStorage[this.$$storeUid];
    };
    Storable.prototype.deleteState = function () {
        this[".listeners"] = null;
        this.dispatchEvent("terminate");
        window.localStorage.removeItem(this.$$storeUid);
        this.storableWillUnmount();
        delete exports.globalStorage[this.$$storeUid];
    };
    Storable.registerClass = function (name) {
        if (!name)
            name = this.name;
        this[".className"] = name;
        exports.storableClasses[name] = this;
        return this;
    };
    Storable.openStorable = function (uid) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        try {
            var obj = void 0, data = void 0;
            if (uid) {
                obj = exports.globalStorage[uid];
                if (obj)
                    return obj;
                data = readStorageData(uid);
            }
            else {
                uid = generateUid();
            }
            // Create object from stored data
            var dataStorableClass = data ? exports.storableClasses[data[".className"]] : null;
            if (dataStorableClass) {
                obj = new (dataStorableClass.bind.apply(dataStorableClass, [void 0].concat(args)))();
                if (!(obj instanceof this)) {
                    console.error("Bad stored className '" + data[".className"] + "' for class " + this.name);
                    obj = null;
                }
            }
            // Create object from this class
            if (!obj) {
                var defaultStorableClass = exports.storableClasses[this[".className"]];
                if (defaultStorableClass) {
                    obj = new (defaultStorableClass.bind.apply(defaultStorableClass, [void 0].concat(args)))();
                    if (!(obj instanceof this)) {
                        console.error("Mis registered storable for class " + this.name);
                        return null;
                    }
                }
                else {
                    return null;
                }
            }
            // Initiate object
            obj.$$storeUid = uid;
            exports.globalStorage[uid] = obj;
            obj.storableShouldInitiate.apply(obj, [data].concat(args));
            return obj;
        }
        catch (e) {
            console.error(e);
        }
        return null;
    };
    return Storable;
}(listenable_1.default));
exports.default = Storable;
function getAllStorable() {
    return exports.globalStorage;
}
exports.getAllStorable = getAllStorable;
function saveAllStorable() {
    for (var uid in exports.globalStorage) {
        exports.globalStorage[uid].forceSave();
    }
}
exports.saveAllStorable = saveAllStorable;
function readStorageData(uid) {
    try {
        var bytes = window.localStorage.getItem(uid);
        if (bytes) {
            var data = JSON.parse(bytes);
            var className = data[".className"];
            if (!className || !exports.storableClasses[className])
                throw new Error("Invalid storable data");
            return data;
        }
    }
    catch (e) {
        window.localStorage.removeItem(uid);
    }
    return null;
}
//window.addEventListener('beforeunload', saveAllStorable)
window.addEventListener('unload', saveAllStorable);
