"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var event_utils_1 = require("./event.utils");
var draggedZone = null;
var dropSuggested = null;
var dropRegistry = [];
function SelectDropSuggested(zone) {
    if (dropSuggested) {
        dropSuggested.unselect();
    }
    zone.select();
    dropSuggested = zone;
}
function UnselectDropSuggested(zone) {
    zone.unselect();
    if (dropSuggested === zone) {
        dropSuggested = null;
    }
}
function RegisterDropZone(zone) {
    zone.zoneId = dropRegistry.length;
    dropRegistry.push(zone);
}
function UnregisterDropZone(zone) {
    var lzone = dropRegistry[dropRegistry.length - 1];
    lzone.zoneId = zone.zoneId;
    zone.zoneId = undefined;
    dropRegistry[lzone.zoneId] = lzone;
    dropRegistry.pop();
}
function FocusDropZone(data) {
    dropRegistry.forEach(function (x) { return x.activate(data); });
}
function BlurDropZone() {
    dropRegistry.forEach(function (x) { return x.deactivate(); });
}
function objectToDataTransfert(obj, dataTransfer) {
    Object.keys(obj).forEach(function (key) {
        if (key !== "dragImage") {
            dataTransfer.setData(key, JSON.stringify(obj[key]));
        }
    });
    if (obj.dragImage) {
        dataTransfer.setDragImage(obj.dragImage.element, obj.dragImage.left || 0, obj.dragImage.top || 0);
    }
}
function dataTransfertToObject(dataTransfer) {
    var obj = {};
    dataTransfer.types.forEach(function (key) {
        try {
            obj[key] = JSON.parse(dataTransfer.getData(key));
        }
        catch (e) {
            // Nothing
        }
    });
    obj.files = dataTransfer.files;
    return obj;
}
var DragZone = /** @class */ (function (_super) {
    __extends(DragZone, _super);
    function DragZone() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleDragStart = function (evt) {
            var bag = _this.props.onDragStart && _this.props.onDragStart(evt);
            if (bag) {
                draggedZone = _this;
                evt.stopPropagation();
                objectToDataTransfert(bag, evt.dataTransfer);
                FocusDropZone(bag);
            }
        };
        _this.handleDragEnd = function (evt) {
            evt.stopPropagation();
            _this.dragComplete();
            BlurDropZone();
        };
        return _this;
    }
    DragZone.prototype.dragComplete = function (data) {
        if (draggedZone === this) {
            this.props.onDragEnd && this.props.onDragEnd(data || {});
        }
        draggedZone = null;
    };
    DragZone.prototype.render = function () {
        var _a = this.props, onDragStart = _a.onDragStart, onDragEnd = _a.onDragEnd, otherProps = __rest(_a, ["onDragStart", "onDragEnd"]);
        return (react_1.default.createElement("div", __assign({ draggable: true, onMouseDown: event_utils_1.stopPropagation, onDragStart: this.handleDragStart, onDragEnd: this.handleDragEnd }, otherProps)));
    };
    return DragZone;
}(react_1.default.Component));
exports.DragZone = DragZone;
var DropZone = /** @class */ (function (_super) {
    __extends(DropZone, _super);
    function DropZone() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.zoneId = undefined;
        _this.active = false;
        _this.componentWillMount = function () {
            RegisterDropZone(_this);
        };
        _this.componentWillUnmount = function () {
            UnregisterDropZone(_this);
        };
        _this.handleDrop = function (evt) {
            if (dropSuggested === _this) {
                evt.stopPropagation();
                evt.preventDefault();
                UnselectDropSuggested(_this);
                var rect = evt.currentTarget.getBoundingClientRect();
                var x = evt.clientX - rect.left;
                var y = evt.clientY - rect.top;
                var bag = dataTransfertToObject(evt.dataTransfer);
                try {
                    var acknowledgment = _this.props.onDrop && _this.props.onDrop(bag, x, y, evt);
                    draggedZone && draggedZone.dragComplete(acknowledgment);
                }
                catch (e) {
                    console.error(e);
                }
            }
        };
        _this.handleDragOver = function (evt) {
            if (dropSuggested === _this || !_this.props.onDropMatch || _this.props.onDropMatch(evt.dataTransfer.types)) {
                evt.preventDefault();
                evt.stopPropagation();
                SelectDropSuggested(_this);
            }
        };
        _this.handleDragEnter = function (evt) {
            if (dropSuggested === _this || !_this.props.onDropMatch || _this.props.onDropMatch(evt.dataTransfer.types)) {
                evt.preventDefault();
                evt.stopPropagation();
                if (evt.currentTarget === _this.refs.zone) {
                    SelectDropSuggested(_this);
                }
            }
        };
        _this.handleDragLeave = function (evt) {
            if (dropSuggested === _this) {
                evt.preventDefault();
                evt.stopPropagation();
                UnselectDropSuggested(_this);
            }
        };
        return _this;
    }
    DropZone.prototype.activate = function (data) {
        var props = this.props;
        if (!props.isDroppable || props.isDroppable(data)) {
            this.active = true;
        }
        else {
            this.active = false;
        }
        var dom = this.refs.zone;
        if (this.active && props.highlightClassName) {
            dom.className = props.className + " " + props.highlightClassName;
        }
        else {
            dom.className = props.className;
        }
    };
    DropZone.prototype.deactivate = function () {
        if (this.active) {
            var dom = this.refs.zone;
            dom.className = this.props.className;
            this.active = false;
        }
    };
    DropZone.prototype.select = function () {
        var props = this.props;
        var dom = this.refs.zone;
        if (this.active && props.selectedClassName) {
            if (props.highlightClassName) {
                dom.className = props.className + " " + props.highlightClassName + " " + props.selectedClassName;
            }
            else {
                dom.className = props.className + " " + props.selectedClassName;
            }
        }
        else {
            dom.className = props.className;
        }
        props.onSelect && props.onSelect();
    };
    DropZone.prototype.unselect = function () {
        var props = this.props;
        var dom = this.refs.zone;
        if (this.active && props.highlightClassName) {
            dom.className = props.className + " " + props.highlightClassName;
        }
        else {
            dom.className = props.className;
        }
        props.onUnselect && props.onUnselect();
    };
    DropZone.prototype.render = function () {
        var _a = this.props, onSelect = _a.onSelect, onUnselect = _a.onUnselect, onDrop = _a.onDrop, onDropMatch = _a.onDropMatch, selectedClassName = _a.selectedClassName, highlightClassName = _a.highlightClassName, isDroppable = _a.isDroppable, otherProps = __rest(_a, ["onSelect", "onUnselect", "onDrop", "onDropMatch", "selectedClassName", "highlightClassName", "isDroppable"]);
        return (react_1.default.createElement("div", __assign({ ref: "zone", className: this.props.className, onDrop: this.handleDrop, onDragOver: this.handleDragOver, onDragEnter: this.handleDragEnter, onDragLeave: this.handleDragLeave }, otherProps)));
    };
    return DropZone;
}(react_1.default.Component));
exports.DropZone = DropZone;
var DragDropZone = /** @class */ (function (_super) {
    __extends(DragDropZone, _super);
    function DragDropZone() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleDragStart = function (evt) {
            var bag = _this.props.onDragStart && _this.props.onDragStart(evt);
            if (bag) {
                evt.stopPropagation();
                objectToDataTransfert(bag, evt.dataTransfer);
                FocusDropZone(bag);
            }
        };
        _this.handleDragEnd = function (evt) {
            evt.stopPropagation();
            _this.props.onDragEnd && _this.props.onDragEnd(evt);
            BlurDropZone();
        };
        return _this;
    }
    DragDropZone.prototype.render = function () {
        var _a = this.props, onSelect = _a.onSelect, onUnselect = _a.onUnselect, onDrop = _a.onDrop, onDropMatch = _a.onDropMatch, selectedClassName = _a.selectedClassName, highlightClassName = _a.highlightClassName, isDroppable = _a.isDroppable, onDragStart = _a.onDragStart, onDragEnd = _a.onDragEnd, otherProps = __rest(_a, ["onSelect", "onUnselect", "onDrop", "onDropMatch", "selectedClassName", "highlightClassName", "isDroppable", "onDragStart", "onDragEnd"]);
        return (react_1.default.createElement("div", __assign({ ref: "zone", draggable: true, className: this.props.className, onMouseDown: event_utils_1.stopPropagation, onDragStart: this.handleDragStart, onDragEnd: this.handleDragEnd, onDrop: this.handleDrop, onDragOver: this.handleDragOver, onDragEnter: this.handleDragEnter, onDragLeave: this.handleDragLeave }, otherProps)));
    };
    return DragDropZone;
}(DropZone));
exports.DragDropZone = DragDropZone;
