"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var ResizableBorder_1 = require("./ResizableBorder");
var defaultArray = [];
var Split = /** @class */ (function (_super) {
    __extends(Split, _super);
    function Split() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.sizes = [];
        _this.modes = [];
        _this.handleResize = function (index) { return function (delta) {
            var _a = _this, sizes = _a.sizes, modes = _a.modes;
            if (modes[index] > 0) {
                var csize = _this.refs[index.toString()].clientHeight;
                // Update sizes
                var size = sizes[index];
                delta = Math.max(size, 1) * delta / Math.max(csize, 1);
                sizes[index] += delta;
                sizes[index + 1] -= delta;
                _this.applySizes();
            }
        }; };
        _this.handleCollapse = function (index) { return function () {
            var modes = _this.modes;
            modes[index] = -modes[index];
            _this.applySizes();
        }; };
        return _this;
    }
    Split.prototype.componentWillMount = function () {
        this.updateProps(this.props);
    };
    Split.prototype.componentWillReceiveProps = function (nextProps) {
        this.updateProps(nextProps);
    };
    Split.prototype.transformDelta = function (e) {
        return e.deltaY;
    };
    Split.prototype.updateProps = function (props) {
        var hasChanged;
        var prevItems = this.items || defaultArray;
        var nextItems = props.items || defaultArray;
        this.items = nextItems;
        // Check items count
        hasChanged = (nextItems.length !== prevItems.length);
        // Update sizes and modes
        for (var i = 0; i < nextItems.length; i++) {
            var item = nextItems[i];
            var prevItem = prevItems[i];
            if (!prevItem || item.size !== prevItem.size) {
                hasChanged = true;
                if (item.size === undefined) {
                    this.sizes[i] = 0;
                    this.modes[i] = 0;
                }
                else {
                    this.sizes[i] = item.size;
                    if (this.modes[i] === undefined)
                        this.modes[i] = 1;
                }
            }
        }
        // Update styles and normalize sizes
        hasChanged && this.applySizes();
    };
    Split.prototype.applySizes = function () {
        var _a = this, items = _a.items, sizes = _a.sizes, modes = _a.modes;
        this.styles = [];
        this.sizes = [];
        // Accumulate sizes
        var len = 0;
        for (var i = 0; i < items.length; i++) {
            var sz = this.sizes[i] = sizes[i] || items[i].size;
            if (sz > 0)
                len += sz;
        }
        // Normalize sizes
        var factor = len ? 100 / len : 1;
        for (var i = 0; i < items.length; i++) {
            var style = this.styles[i] = __assign({}, styles.content);
            var mode = modes[i], item = items[i];
            this.sizes[i] *= factor;
            if (mode > 0) {
                style.overflow = item.overflow || "auto";
                style.flex = sizes[i];
            }
            else if (mode < 0) {
                style.overflow = item.overflow || "auto";
                style.flex = 0;
                style.height = 0;
            }
            else {
                style.flex = (i < items.length - 1) ? 0 : 1;
                style.height = "auto";
            }
        }
        // Update
        this.forceUpdate();
    };
    Split.prototype.render = function () {
        var items = this.props.items;
        var contents = [];
        if (!items || !Array.isArray(items)) {
            console.error("Empty Collapsible Window is useless.");
            return null;
        }
        var lastIndex = items.length - 1;
        for (var i = 0; i <= lastIndex; i++) {
            var item = items[i];
            // Render header
            if (item.header) {
                contents.push(react_1.default.createElement("span", { key: contents.length, style: styles.header, onClick: this.handleCollapse(i) },
                    this.modes[i] <= 0
                        ? react_1.default.createElement("span", { className: "fa fa-fw fa-caret-right padding-xs" })
                        : react_1.default.createElement("span", { className: "fa fa-fw fa-caret-down padding-xs" }),
                    item.header));
            }
            // Render content
            contents.push(react_1.default.createElement("div", { key: contents.length, ref: i.toString(), style: this.styles[i] }, item.content));
            // Render resizer
            if (i < lastIndex) {
                contents.push(react_1.default.createElement(ResizableBorder_1.ResizableBorder, { key: contents.length, onResize: this.handleResize(i), transformDelta: this.transformDelta }));
            }
        }
        return (react_1.default.createElement("div", { style: styles.panel }, contents));
    };
    return Split;
}(react_1.Component));
exports.default = Split;
var styles = {
    header: {
        display: "flex",
        flexDirection: "row",
        whiteSpace: "nowrap",
        cursor: "pointer",
        backgroundColor: "#234",
        borderBottom: "solid 1px #012",
        color: "#89a",
        overflow: "hidden",
    },
    content: {
        backgroundColor: "#fff",
    },
    panel: {
        display: "flex",
        height: "100%",
        maxHeight: "100%",
        minHeight: "100%",
        maxWidth: "100%",
        minWidth: "100%",
        flexDirection: "column",
        backgroundColor: "#234",
    }
};
