"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function stopEvent(e) {
    if (e.stopPropagation)
        e.stopPropagation();
    if (e.preventDefault)
        e.preventDefault();
    return false;
}
exports.stopEvent = stopEvent;
function stopPropagation(e) {
    if (e.stopPropagation)
        e.stopPropagation();
    return false;
}
exports.stopPropagation = stopPropagation;
var HtmlScrollingReaction = /** @class */ (function () {
    function HtmlScrollingReaction(element, event, onMouseScroll) {
        stopEvent(event);
        this.element = element;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        this.scrollX = element.scrollLeft;
        this.scrollY = element.scrollTop;
        this.onMouseScroll = onMouseScroll;
        setCurrentGrabReaction(this);
    }
    HtmlScrollingReaction.prototype.scroll = function () {
        this.element.scrollLeft = this.scrollX;
        this.element.scrollTop = this.scrollY;
    };
    HtmlScrollingReaction.prototype.handleMouseMove = function (event) {
        stopEvent(event);
        this.scrollX += this.mouseX - event.screenX;
        this.scrollY += this.mouseY - event.screenY;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        if (this.onMouseScroll)
            this.onMouseScroll(this);
        else
            this.scroll();
    };
    HtmlScrollingReaction.prototype.handleCompleted = function () {
        removeCurrentGrabReaction(this);
    };
    return HtmlScrollingReaction;
}());
exports.HtmlScrollingReaction = HtmlScrollingReaction;
var HtmlGrabReaction = /** @class */ (function () {
    function HtmlGrabReaction(element, event, onMouseGrab, onMouseRelease) {
        stopEvent(event);
        this.element = element;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        this.left = element.offsetLeft;
        this.top = element.offsetTop;
        this.onMouseGrab = onMouseGrab;
        this.onMouseRelease = onMouseRelease;
        setCurrentGrabReaction(this);
    }
    HtmlGrabReaction.prototype.move = function () {
        this.element.style.left = this.left + "px";
        this.element.style.top = this.top + "px";
    };
    HtmlGrabReaction.prototype.handleMouseMove = function (event) {
        stopEvent(event);
        this.deltaX = event.screenX - this.mouseX;
        this.deltaY = event.screenY - this.mouseY;
        this.left += this.deltaX;
        this.top += this.deltaY;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        if (this.onMouseGrab)
            this.onMouseGrab(this);
        else
            this.move();
    };
    HtmlGrabReaction.prototype.handleCompleted = function () {
        if (this.onMouseRelease)
            this.onMouseRelease(this);
        removeCurrentGrabReaction(this);
    };
    return HtmlGrabReaction;
}());
exports.HtmlGrabReaction = HtmlGrabReaction;
var SVGGrabReaction = /** @class */ (function () {
    function SVGGrabReaction(element, event, onMouseGrab) {
        stopEvent(event);
        this.element = element;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        this.x = element.x();
        this.y = element.y();
        this.onMouseGrab = onMouseGrab;
        setCurrentGrabReaction(this);
    }
    SVGGrabReaction.prototype.move = function () {
        this.element.x(this.x);
        this.element.y(this.y);
    };
    SVGGrabReaction.prototype.handleMouseMove = function (event) {
        stopEvent(event);
        this.x += event.screenX - this.mouseX;
        this.y += event.screenY - this.mouseY;
        this.mouseX = event.screenX;
        this.mouseY = event.screenY;
        if (this.onMouseGrab)
            this.onMouseGrab(this);
        else
            this.move();
    };
    SVGGrabReaction.prototype.handleCompleted = function () {
        removeCurrentGrabReaction(this);
    };
    return SVGGrabReaction;
}());
exports.SVGGrabReaction = SVGGrabReaction;
var grabReaction = null;
function setCurrentGrabReaction(reaction) {
    grabReaction && grabReaction.handleCompleted();
    grabReaction = reaction;
}
function removeCurrentGrabReaction(reaction) {
    if (grabReaction === reaction)
        grabReaction = null;
}
window.addEventListener("mousemove", function (e) {
    grabReaction && grabReaction.handleMouseMove(e);
});
window.addEventListener("mouseup", function () {
    grabReaction && grabReaction.handleCompleted();
});
