"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importDefault(require("react"));
var react_dom_1 = __importDefault(require("react-dom"));
var computeEdgeBox_1 = require("../computeEdgeBox");
require("./style.css");
var stack = [];
var stopableEvents = ["click", "dbclick", "contextmenu"];
var defaultClassName = "";
var defaultStyle = {
    border: "1px solid #abc",
    backgroundColor: "#fff",
    padding: "5px 0 5px 0",
    minWidth: "150px",
    overflow: "auto",
};
exports.Menu = {
    Item: function (props) {
        var _this = this;
        var icon = props.icon, title = props.title, children = props.children, onClick = props.onClick;
        var onMouseEnter, onMouseLeave;
        if (children) {
            var closeCallback_1;
            onMouseEnter = function (e) {
                openContextualMenu(_this, e.currentTarget, function (f) {
                    closeCallback_1 = f;
                    return children;
                });
            };
            onMouseLeave = function () {
                closeCallback_1 && closeCallback_1();
            };
            if (!onClick) {
                onClick = function (e) {
                    openContextualMenu(_this, e.currentTarget, function () { return children; });
                };
            }
        }
        return react_1.default.createElement("div", { className: "raf-menu-item", onClick: onClick, onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave },
            react_1.default.createElement("div", null, icon),
            react_1.default.createElement("div", null, title));
    },
    Separator: function () {
        return react_1.default.createElement("div", { className: "raf-menu-separator" });
    },
};
function setDefaultMenuStyle(className, style) {
    defaultStyle = style || defaultStyle;
    defaultClassName = className || defaultClassName;
}
exports.setDefaultMenuStyle = setDefaultMenuStyle;
function openContextualMenu(parent, target, renderer, position, className, style) {
    console.assert(renderer instanceof Function);
    var resolve = null;
    // Determine tracked element
    var tracked;
    if (target instanceof Object) {
        if (target instanceof Element) {
            tracked = target;
        }
        else if (target["currentTarget"] instanceof Element) {
            if (stopableEvents.indexOf(target["type"]) >= 0) {
                if (target["stopPropagation"] instanceof Function)
                    target["stopPropagation"]();
                if (target["preventDefault"] instanceof Function)
                    target["preventDefault"]();
            }
            tracked = target["currentTarget"];
        }
        else {
            tracked = react_dom_1.default.findDOMNode(target);
        }
    }
    else {
        throw new Error("target is required");
    }
    // Purge top of stack popup
    var stackIndex = 0;
    while (stackIndex < stack.length) {
        if (!stack[stackIndex].node.contains(tracked)) {
            stack[stackIndex].close();
            break;
        }
        stackIndex++;
    }
    // Create popup node
    var node = document.createElement("div");
    node.className = className || defaultClassName;
    Object.assign(node.style, style || defaultStyle);
    node.style.visibility = "hidden";
    node.style.position = "fixed";
    node.style.zIndex = ((stackIndex + 1) * 100).toString();
    function clickOutside(e) {
        if (node) {
            for (var i = stackIndex; i < stack.length; i++) {
                if (stack[i].node.contains(e.target))
                    return;
            }
            close();
        }
    }
    function updatePosition() {
        if (node) {
            computeEdgeBox_1.computeEdgeBoxDOM(position, node, tracked);
            node.style.visibility = "visible";
            setTimeout(updatePosition, 25);
        }
    }
    function close(value) {
        if (node) {
            // Remove popup
            document.body.removeEventListener("mousedown", clickOutside);
            document.body.removeChild(node);
            react_dom_1.default.unmountComponentAtNode(node);
            node = null;
            // Close sub popup when not the top of stack
            if (stackIndex < stack.length - 1) {
                stack[stackIndex + 1].close();
            }
            stack.pop();
            // Resolve promise
            resolve && resolve(value);
        }
    }
    // Append popup in document on top of stack
    document.body.appendChild(node);
    document.body.addEventListener("mousedown", clickOutside);
    computeEdgeBox_1.computeEdgeBoxDOM(position, node, tracked, document.body);
    stack.push({ node: node, close: close });
    // Render popup on node
    react_dom_1.default.render(renderer(close), node, updatePosition);
    // Make the promise
    var promise = new Promise(function (_resolve) {
        if (node)
            resolve = _resolve;
        else
            resolve();
    });
    promise["close"] = close;
    return promise;
}
exports.default = openContextualMenu;
